package com.woosignal.flutter_stripe_payment

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.Button

import com.stripe.android.*
import com.stripe.android.model.*
import com.stripe.android.view.CardMultilineWidget

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar


class FlutterStripePaymentPlugin(private var context: Context, private var activity: Activity) : MethodCallHandler {
  
  private lateinit var stripe: Stripe

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "woosignal_stripe")
      channel.setMethodCallHandler(FlutterStripePaymentPlugin(registrar.context(), registrar.activity()))
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {

    val arguments = call.arguments as? Map<*, *>

    when (call.method) {
        "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
        "setStripeSettings" -> {
          val stripePublishableKey = arguments?.get("stripePublishableKey") as? String
          val stripeAccount = arguments?.get("stripeAccount") as? String
          this.setStripeSettings(result, stripePublishableKey as String, stripeAccount as String)
        }
        "addPaymentMethod" -> {
          this.addPaymentMethod(result)
        }
        "confirmPaymentIntent" -> {
          val clientSecret = arguments?.get("clientSecret") as? String
          val paymentMethodId = arguments?.get("paymentMethodId") as? String
          val amount = arguments?.get("amount") as? Double
          this.confirmPaymentIntent(paymentMethodId!!, clientSecret!!, amount!!, result)
        }
      "setupPaymentIntent" -> {
        val clientSecret = arguments?.get("clientSecret") as? String
        val paymentMethodId = arguments?.get("paymentMethodId") as? String
        val amount = arguments?.get("amount") as? Double
        this.setupPaymentIntent(paymentMethodId!!, clientSecret!!, amount!!, result)
      }
        else -> {
          result.notImplemented()
        }
    }
  }

  private fun setStripeSettings(result: Result, stripePublishableKey: String, stripeAccount: String)
  {
    PaymentConfiguration.init(context, stripePublishableKey)
    stripe = Stripe(context, PaymentConfiguration.getInstance(context).publishableKey,stripeAccount)
    StripeAccountHolder.setStripeAccount(stripeAccount)
    result.success(true)
  }

  private fun addPaymentMethod(result: Result)
  {
    TempHolder.setData(result)
    val intent = Intent(context, PaymentActivity::class.java)
    intent.putExtra("showPaymentForm", true)
    activity.startActivity(intent)
  }

  private fun confirmPaymentIntent(paymentMethodId: String, clientSecret: String, amount: Double?, result: Result)
  {
    TempHolder.setData(result)
    TempHolder.setPaymentData(PaymentData(clientSecret, amount, "", paymentMethodId))
    val intent = Intent(context, PaymentActivity::class.java)
    intent.putExtra("showPaymentForm", false)
    intent.putExtra("confirmPaymentIntent", true)
    activity.startActivity(intent)
  }

  private fun setupPaymentIntent(paymentMethodId: String, clientSecret: String, amount:Double?, result: Result)
  {
    TempHolder.setData(result)
    TempHolder.setPaymentData(PaymentData(clientSecret, amount, "", paymentMethodId))
    val intent = Intent(context, PaymentActivity::class.java)
    intent.putExtra("setupPaymentIntent", true)
    activity.startActivity(intent)
  }

}

class PaymentActivity : Activity()
{
  private lateinit var flutterResult: Result
  private lateinit var stripe: Stripe
  private lateinit var createPaymentMethod: Button
  private lateinit var cardInputWidget: CardMultilineWidget

  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)

    if (StripeAccountHolder.getHolderStripeAccount() != "") {
      stripe = Stripe(this,
              PaymentConfiguration.getInstance(this).publishableKey, StripeAccountHolder.getHolderStripeAccount())
    } else {
      stripe = Stripe(this,
              PaymentConfiguration.getInstance(this).publishableKey)
    }

    val showPaymentForm = intent.getBooleanExtra("showPaymentForm", false)
    val confirmPaymentIntent = intent.getBooleanExtra("confirmPaymentIntent", false)
    val setupPaymentIntent = intent.getBooleanExtra("setupPaymentIntent", false)

   if(showPaymentForm)
   {
     val uiCustomization = PaymentAuthConfig.Stripe3ds2UiCustomization.Builder()
             .build()
     PaymentAuthConfig.init(PaymentAuthConfig.Builder()
             .set3ds2Config(PaymentAuthConfig.Stripe3ds2Config.Builder()
                     .setTimeout(5)
                     .setUiCustomization(uiCustomization)
                     .build())
             .build())
     setContentView(R.layout.card_input_widget)
     cardInputWidget = findViewById(R.id.card_input_widget)
     createPaymentMethod = findViewById(R.id.btn_create_payment_method)
     createPaymentMethod.setOnClickListener {

       val card = cardInputWidget.card
       if (card != null) {
         findViewById<View>(R.id.mProgressBar)?.visibility = View.VISIBLE
         findViewById<View>(R.id.btn_create_payment_method)?.visibility = View.GONE
         createPaymentMethod(card)
       }
       else
       {
         flutterResult = TempHolder.getResult() as Result
       }
     }
   }

    if(confirmPaymentIntent)
    {
      val data = TempHolder.getPaymentData()
      val params = ConfirmPaymentIntentParams.createWithPaymentMethodId(data!!.paymentMethodId, data.clientSecret, "stripe://create_payment_intent_return")
      stripe.confirmPayment(this, params)
    }

    if(setupPaymentIntent)
    {
      val data = TempHolder.getPaymentData()
      val params = ConfirmSetupIntentParams.create(data!!.paymentMethodId, data.clientSecret, "stripe://create_payment_intent_return")
      stripe.confirmSetupIntent(this, params)
    }

  }

  private fun createPaymentMethod(card: Card) {

    flutterResult = TempHolder.getResult() as Result

    val paymentMethodParamsCard = card.toPaymentMethodParamsCard()
    val paymentMethodCreateParams =
            PaymentMethodCreateParams.create(paymentMethodParamsCard, PaymentMethod.BillingDetails.Builder().build())

    stripe.createPaymentMethod(
            paymentMethodCreateParams,
            null,
            StripeAccountHolder.getHolderStripeAccount(),
            object : ApiResultCallback<PaymentMethod> {

              override fun onSuccess(result: PaymentMethod) {
                findViewById<View>(R.id.mProgressBar)?.visibility = View.GONE
                findViewById<View>(R.id.btn_create_payment_method)?.visibility = View.GONE
                findViewById<View>(R.id.card_input_widget)?.visibility = View.GONE

                if (result.id != null) {
                  val paymentResponse = mapOf("status" to "succeeded", "paymentMethodId" to (result.id ?: "") )
                  flutterResult.success(paymentResponse)
                }
                finish()
              }

              override fun onError(e: Exception) {
                findViewById<View>(R.id.card_input_widget)?.visibility = View.GONE
                findViewById<View>(R.id.mProgressBar)?.visibility = View.GONE
                findViewById<View>(R.id.btn_create_payment_method)?.visibility = View.VISIBLE
                val paymentResponse = mapOf("status" to "failed")
                flutterResult.success(paymentResponse)
                finish()
              }

            })

  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent) {
    super.onActivityResult(requestCode, resultCode, data)
    flutterResult = TempHolder.getResult() as Result

    stripe.onPaymentResult(requestCode, data,
            object : ApiResultCallback<PaymentIntentResult> {
              override fun onSuccess(result: PaymentIntentResult) {

                val paymentIntent = result.intent
                val status = paymentIntent.status
                if (status == StripeIntent.Status.Succeeded) {

                  val paymentResponse = mapOf("status" to "succeeded", "paymentIntentId" to (paymentIntent.id ?: "") )
                  flutterResult.success(paymentResponse)
                  finish()
                }
                else if (StripeIntent.Status.RequiresPaymentMethod == status) {
                  // attempt authentication again or
                  // ask for a new Payment Method
                }
                else if(status == StripeIntent.Status.Canceled)
                {
                  val paymentResponse = mapOf("status" to "canceled" )
                  flutterResult.success(paymentResponse)
                }
                finish()
              }

              override fun onError(e: Exception) {
                val paymentResponse = mapOf("status" to "failed", "errorMessage" to e.localizedMessage )
                flutterResult.success(paymentResponse)
                finish()
              }
            })

    stripe.onSetupResult(requestCode, data,
            object : ApiResultCallback<SetupIntentResult> {
              override fun onSuccess(result: SetupIntentResult) {

                val setupIntent = result.intent
                val status = setupIntent.status
                if (status == StripeIntent.Status.Succeeded) {

                  val paymentResponse = mapOf("status" to "succeeded", "paymentIntentId" to (setupIntent.id ?: "") )
                  flutterResult.success(paymentResponse)
                  finish()
                } else if (setupIntent.requiresConfirmation()) {
                  // handle confirmation
                  finish()
                }
              }

              override fun onError(e: Exception) {
                val paymentResponse = mapOf("status" to "failed", "errorMessage" to e.localizedMessage )
                flutterResult.success(paymentResponse)
                finish()
              }
            })
  }

}


object TempHolder {
  private var flutterResult: Result? = null
  private var paymentData: PaymentData? = null
  fun getResult(): Result? {
    return flutterResult
  }

  fun setData(result: Result){
    flutterResult = result
  }

  fun getPaymentData(): PaymentData?{
    return paymentData
  }

  fun setPaymentData(data: PaymentData)
  {
    paymentData = data
  }
}

class PaymentData(clientSecret: String,  amount: Double?, paymentIntentId: String, paymentMethodId: String)
{
  var paymentMethodId: String
  var clientSecret: String
  var paymentIntentId: String? = null
  var amount: Double? = null

  init {
    this.paymentMethodId = paymentMethodId
    this.clientSecret = clientSecret
    this.paymentIntentId = paymentIntentId
    this.amount = amount
  }
}

object StripeAccountHolder {
  private var stripeAccount: String = ""

  fun setStripeAccount(stripeAccount: String) {
    this.stripeAccount = stripeAccount
  }

  fun getHolderStripeAccount() : String {
    return this.stripeAccount
  }
}